// Copyright (C) 2023, AMD Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#pragma once

#include <mutex>
#include <functional>
#include <condition_variable>
#include "Exception.h"
#include "XmaLog.h"
#ifdef __cplusplus
extern "C" {
#endif
#include "vpi_buffer.h"
#ifdef __cplusplus
}
#endif

namespace xma {

#define SD_BUFFER_POOL_AUTO_EXTEND_DEFAULT_SIZE 16
#define SD_BUFFER_POOL_AUTO_EXTEND_SIZE 1

//////////////////////////////////////////////////////////////////////////////
using side_data_buffer_t = std::shared_ptr<void>; // shared pointer to VpiSidedataHandle

class SDBufferPool {
public:
  SDBufferPool(VpiMemHandle mem, LogPtr log, enum XmaFrameSideDataType type, uint32_t num, uint32_t size, uint32_t flag = RC_SIDE_EN | EP_SIDE_EN);
  virtual ~SDBufferPool();

  int32_t SetLog(LogPtr log);

  side_data_buffer_t GetBuffer(uint32_t size);
  int32_t            Extend(uint32_t num);

protected:
  void InitData();

protected:
  using lock_t = std::unique_lock<std::recursive_mutex>;
  using pool_t = std::unique_ptr<VpiBufferPool, std::function<void(VpiBufferPool* pool)>>;
  std::recursive_mutex m_mutex;
  pool_t               m_pool;
  XmaFrameSideDataType m_type;
  uint32_t             m_size = 0;
  LogPtr               m_log;
};

using SDBufferPoolPtr = std::shared_ptr<SDBufferPool>;

}; // Namespace xma
